#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	DivideByMaterial
#      By:	sdmitch
#   Usage:	To sub-divide a face with given material
#	 Note:	If sub-division spans a wall opening, it will sometimes cover and close the opening.
#    Date:	May 2011
#------------------------------------------------------------------------------------------------

module DivideMaterial
	
	def self.select_materials
		mod = Sketchup.active_model
		ent = mod.active_entities
		sel = mod.selection
		if sel.empty?
			UI.messagebox "Nothing selected"
			return nil
		end
		materials = mod.materials
		mat1=[];materials.each{|m|mat1<<m};# get all materials in model
		mat2=[];materials.each{|m|mat2<<m.name}; # and their names
		matn=mat2.join("|"); # create an material names list
		ans=UI.inputbox(["Material:","Max Size:"],[nil,1.m],[matn], "Divide By Material")
		if !ans then return end
		fn=ans[0].to_s;d=ans[1].to_l; fi=mat2.index(fn)
		mod.start_operation "Divide Face"
		faces=[]; points=[]; i=0; j=0
		sel.each {|f| faces.push f if f.is_a? Sketchup::Face}
		faces.each {|f| points[i]=f.outer_loop.vertices.collect{|v| v.position};i+=1}
		grp = ent.add_group; # create a group 
		@ge = grp.entities;   # for c_line entities
		faces.each do |e|
			unless e.material == nil
				if e.material == mat1[fi]
					self.divide_face(e,points[j],d)
				end
			end
			unless e.back_material == nil
				if e.back_material == mat1[fi]
					self.divide_face(e,points[j],d)
				end
			end
			j+=1
		end
#		convert construction lines to edges			
		@ge.each {|e| edg=ent.add_line(e.start,e.end)}
		ent.erase_entities grp
		mod.commit_operation
	end
	
	def self.divide_face(face,pts,dmax)
		mod = Sketchup.active_model
		ent = mod.active_entities
		if pts.length > 4
			longest_side = 0.0; #find the longest side
			for i in 0...pts.length
				dist=pts[i-1].distance(pts[i])
				if dist > longest_side
					longest_side = dist
					ndx = i-1
				end
			end
			ndx += pts.length if ndx < 0
			for i in 0...ndx
				tmp=pts.shift
				pts.push tmp
			end
		end
		d1=pts[1].distance(pts[0])
		d2=pts[1].distance(pts[2])
		if (d1 >= dmax && d2 >= dmax)
			xs=(d1/dmax).ceil; ys=(d2/dmax).ceil
			dx=d1/xs;dy=d2/ys
			v1=pts[1].vector_to(pts[0])
			v2=pts[1].vector_to(pts[2])
			edges=[]; edges=face.edges
			
			for i in 1...xs
				p0=pts[1].offset(v1,dx*i)
				last_pt_out = false
				for j in 1..ys
					p1=p0.offset(v2,dy)
					if !(face.classify_point(p1) == Sketchup::Face::PointOutside)
						if !last_pt_out
							edg = @ge.add_cline(p0,p1)
						else
							pt=self.find_intersection(p0,p1,edges)
							if pt
								edg = @ge.add_cline(pt,p1)
							else
								puts "no intersection found between #{p0} and #{p1}"
								ent.add_cpoint(p1)
							end
						end
						last_pt_out = false
					else
						if !last_pt_out
							pt=self.find_intersection(p0,p1,edges)
							if pt
								edg = @ge.add_cline(p0,pt)
							else
								puts "no intersection found between #{p0} and #{p1}"
								ent.add_cpoint(p1)
							end
						end
						last_pt_out = true
					end
					p0=p1
				end
			end
			
			for i in 1...ys
				p0=pts[1].offset(v2,dy*i)
				last_pt_out = false
				for j in 1..xs
					p1=p0.offset(v1,dx)
					if !(face.classify_point(p1) == Sketchup::Face::PointOutside)
						if !last_pt_out
							edg = @ge.add_cline(p0,p1)
						else
							pt=self.find_intersection(p0,p1,edges)
							if pt
								edg = @ge.add_cline(pt,p1)
							else
								puts "no intersection found between #{p0} and #{p1}"
								ent.add_cpoint(p1)
							end
						end
						last_pt_out = false
					else
						if !last_pt_out
							pt=self.find_intersection(p0,p1,edges)
							if pt
								edg = @ge.add_cline(p0,pt)
							else
								puts "no intersection found between #{p0} and #{p1}"
								ent.add_cpoint(p1)
							end
						end
						last_pt_out = true
					end
					p0=p1
				end
			end
		else
			UI.messagebox "Max size is greater than short side of rectangle"
		end
	end
	
	def self.find_intersection(p0,p1,edges)
		for e in edges
			ln=e.line
			pp=p0.project_to_line ln
			if self.is_between(pp,e.start.position,e.end.position)
				if self.is_between(pp,p0,p1)
					return pp
				end
			end
		end
		return nil
	end
	
	def  self.is_between(p0,p1,p2)
	# --------------------------------------------------------------------------------
	#	Determine if point p0 is on the line and between end points p1 and p2
	# --------------------------------------------------------------------------------
		d1=p0.distance p1 
		d2=p0.distance p2 
		d3=p1.distance p2 
		return true if ((d1+d2) == d3)
		return false # p0 is not on line or between p1 and p2
	end
end

if !file_loaded?("DivideByMaterial.rb")
	UI.menu("Plugins").add_item("DivideByMaterial") { DivideMaterial.select_materials }
end

file_loaded("DivideByMaterial.rb")
